from enum import Enum

class VBATypeName(Enum):
    Byte = 'Byte'
    Integer = 'Integer'
    Long = 'Long'
    String = 'String'
    Single = 'Single'
    Double = 'Double'
    Boolean = 'Boolean'
    Variant = 'Variant'