GEOMETRY_METER = 'm'
GEOMETRY_CENTIMETER = 'cm'
GEOMETRY_MILLIMETER = 'mm'
GEOMETRY_MICROMETER = 'um'
GEOMETRY_NANOMETER = 'nm'
GEOMETRY_FEET = 'ft'
GEOMETRY_INCH = 'in'
GEOMETRY_MIL = 'mil'

TIME_FEMTOSECOND = 'fs'
TIME_PICOSECOND = 'ps'
TIME_NANOSECOND = 'ns'
TIME_MICROSECOND = 'us'
TIME_MILLISECOND = 'ms'
TIME_SECOND = 's'

FREQUENCY_HERTZ = 'Hz'
FREQUENCY_KILOHERTZ = 'kHz'
FREQUENCY_MEGAHERTZ = 'MHz'
FREQUENCY_GIGAHERTZ = 'GHz'
FREQUENCY_TERAHERTZ = 'THz'
FREQUENCY_PETAHERTZ = 'PHz'

TEMPERATURE_CELSIUS = 'Celsius'
TEMPERATURE_KELVIN = 'Kelvin'
TEMPERATURE_FAHRENHEIT = 'Fahrenheit'